<ul class="side-nav">
<a href="#" class="close right">X</a>
<li class="title-area">
            <h1><a href="#"><img src="{{
                static_url('img/favicon/favicon-32x32.png')}}"/>&nbsp;PoWA</a></h1>
</li>


{% if menu %}
{% raw menu.render(handler) %}
{% end %}
</ul>
